//This is a simple program showing how to call oo4o api from a mulithreaded application.
//Note that every thread has its own OStartup() and OShutdown() routines.

#include "windows.h"
#include "stdio.h"
#include "iostream.h"
#include "oracl.h"

DWORD WINAPI browse_dyn();

int nthreads = 1;

void main(int argc, char **argv)
{

	HANDLE	aThreads[100];
	DWORD	aThreadIds[100];

	if (argc == 2)
		nthreads = atoi(argv[1]);  //# of threads
	else 
	{
		cout << "Usage: multhread <# of threads (<=100)> " << endl;
		exit(1);
	}

	
	//Now spawn threads and call browse_dyn function
	int i;
	for (i = 0; i < nthreads; i++)
	{
			
			aThreads[i] = CreateThread(NULL, 
						0, 
						(LPTHREAD_START_ROUTINE) browse_dyn, 
						(LPVOID)NULL , 
						0, 
						&aThreadIds[i]
						);		
	}


	int j = 0;

	// wait for all created threads to complete their execution
	for (j = 0; j < i; j++)
	{

		DWORD	r = WaitForSingleObject(aThreads[j], INFINITE);


		switch (r)
		{
			case WAIT_TIMEOUT:
				cout << "WAIT_TIMEOUT ...\n";
				break;
			case WAIT_OBJECT_0:
				cout <<"WAIT_OBJECT_0 ...\n";
				break;
			case WAIT_ABANDONED_0:
				cout <<"WAIT_ABONDONED_0 ...\n";
				break;
			case WAIT_FAILED:
				cout <<"WAIT_FAILED ...\n";
				break;
			default:
				LPVOID lpMsgBuf;
				FormatMessage( 
					FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
					NULL,
					GetLastError(),
					MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
					(LPTSTR) &lpMsgBuf,
					0,
					NULL 
					);

				cout <<lpMsgBuf ;
	}
	} //end for
	
	

}


DWORD WINAPI browse_dyn( )
{
	cout << "In the thread" << GetCurrentThreadId();

	// OStartup() should be called with every thread invocation	
	// OSTARTUP_MULTITHREADED option should be passed 
	OStartup(OSTARTUP_MULTITHREADED);

	// create session object for each thread. This gives maximum 
	// concurrency to the thread execution. This is also useful when OO4O
	// error reported on session object for one thread cannot be seen by
	// another thread.
	OSession osess ;
	osess.Open();
	
	if ( ! osess.IsOpen() )
	{
		cout <<  "Session not opened: Error: " << osess.GetErrorText() << endl;
		
		osess.Close();
		OShutdown();
		exit(1);
	}

	ODatabase odb;
	odb.Open(osess, "exampledb", "scott", "tiger");
	if ( ! odb.IsOpen() )
	{ 
		cout <<  "Database not opened: " << odb.GetErrorText() << endl;
		odb.Close();
		return(1);
	}

	ODynaset odyn;
	odyn.Open(odb, "select count(*) from emp");
	int isopen = odyn.IsOpen();
	if (isopen)
	{
		int res =0;
		odyn.GetFieldValue(0, &res);
		cout << "# of rows in emp dynaset = " << res  << endl;
	}
	else
	{
		cout <<  "Dynaset not opened: Error:" << odb.GetServerErrorText() << endl;
	}
 

	odyn.Close();
	odb.Close();
	osess.Close();
	OShutdown();
	cout << "Thread Id " << GetCurrentThreadId() <<  "Exiting browse_dyn() " << GetCurrentThreadId() << endl;
	

	return 0;
}


